<?php

declare(strict_types=1);

namespace FiloBlu\Checkproducts\Helper;

use FiloBlu\Checkproducts\Model\CollectorInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Pricing\Price\FinalPrice;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Store\Model\ScopeInterface;

/**
 * Custom Module Email helper
 */
class Data extends AbstractHelper
{
    /**
     * @var string[]
     */
    const  ALLOWED_TYPES = ['simple', 'configurable'];

    /**
     * @var array[]
     */
    private $attributesToCheck = [];

    /**
     * @return bool
     */
    public function isEnabled(): bool
    {
        return $this->scopeConfig->isSetFlag('filoblu_checkproducts/checkproducts_general/checkproducts_enabled');
    }


    /**
     * @return array
     */
    public function getAllAttributesToCheck()
    {
        $attributes = [];
        foreach ($this->getAttributeCheckAllowedTypes() as $type) {
            $attributes = array_merge($attributes, $this->getAttributesToCheck($type));
        }

        return $attributes;
    }

    /**
     * @return string[]
     */
    public function getAttributeCheckAllowedTypes()
    {
        return self::ALLOWED_TYPES;
    }

    /**
     * @param string $type Type::TYPE_SIMPLE | Configurable::TYPE_CODE
     * @return array
     */
    public function getAttributesToCheck(string $type)
    {
        if (isset($this->attributesToCheck[$type])) {
            return $this->attributesToCheck[$type];
        }

        $this->attributesToCheck[$type] = [];

        $attributes = $this->scopeConfig->getValue(
            "filoblu_checkproducts/checkproducts_$type/checkproducts_{$type}_attribute"
        );

        if (!$attributes) {
            return $this->attributesToCheck[$type];
        }

        $this->attributesToCheck[$type] = explode(',', $attributes);

        return $this->attributesToCheck[$type];
    }

    /**
     * @param $type
     * @return bool
     */
    public function canCheckPrice($type): bool
    {
        return $this->scopeConfig->isSetFlag(
            "filoblu_checkproducts/checkproducts_$type/checkproducts_{$type}_price"
        );
    }

    /**
     * @param $type
     * @return bool
     */
    public function canCheckImage($type): bool
    {
        return $this->scopeConfig->isSetFlag(
            "filoblu_checkproducts/checkproducts_$type/checkproducts_{$type}_image"
        );
    }

    /**
     * @param $type
     * @return bool
     */
    public function canCheckCategories($type): bool
    {
        return $this->scopeConfig->isSetFlag(
            "filoblu_checkproducts/checkproducts_$type/checkproducts_{$type}_categories"
        );
    }

    /**
     * @param \Magento\Catalog\Api\Data\ProductInterface $product
     * @return float
     */
    public function getPrice(ProductInterface $product)
    {
        return (float)$product->getPriceInfo()->getPrice(FinalPrice::PRICE_CODE)->getAmount()->getValue();
    }



}