<?php

namespace FiloBlu\Checkproducts\Helper;

use FiloBlu\Checkproducts\Model\CollectorInterface;
use Magento\Framework\App\Area;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\Translate\Inline\StateInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Custom Module Email helper
 */
class Email extends AbstractHelper
{
    const XML_PATH_EMAIL_TEMPLATE_FIELD = 'filoblu_checkproducts/checkproducts_email/checkproducts_email_template';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $inlineTranslation;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $transportBuilder;

    /**
     * @var string
     */
    protected $temp_id;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        StateInterface $inlineTranslation,
        TransportBuilder $transportBuilder
    ) {
        parent::__construct($context);
        $this->scopeConfig = $context->getScopeConfig();
        $this->storeManager = $storeManager;
        $this->inlineTranslation = $inlineTranslation;
        $this->transportBuilder = $transportBuilder;
    }

    /**
     * @param \FiloBlu\Checkproducts\Model\CollectorInterface $collector
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\MailException
     */
    public function sendReport(CollectorInterface $collector)
    {
        $data = $collector->getCollectedData();

        if(empty($data)) {
            return;
        }

        /* Here we prepare data for our email  */

        $sender = $this->scopeConfig->getValue(
            'filoblu_checkproducts/checkproducts_email/checkproducts_email_sender',
            ScopeInterface::SCOPE_STORE
        );
        if (!isset($sender['name']) || !isset($sender['email'])) {
            $sender = 'general';
        }
        $senderEmail = $this->scopeConfig->getValue(
            "trans_email/ident_$sender/email",
            ScopeInterface::SCOPE_STORE
        );
        $senderName = $this->scopeConfig->getValue(
            "trans_email/ident_$sender/name",
            ScopeInterface::SCOPE_STORE
        );

        $receiversEmail = explode(
            ',',
            $this->scopeConfig->getValue(
                'filoblu_checkproducts/checkproducts_email/checkproducts_email_receiver',
                ScopeInterface::SCOPE_STORE
            )
        );

        $receiversEmail = array_filter($receiversEmail, function ($email) {
            return !empty($email);
        });

        if (empty($receiversEmail)) {
            return;
        }

        /* Sender Detail  */
        $senderInfo = [
            'name'  => $senderName,
            'email' => $senderEmail,
        ];

        /* Assign values for your template variables  */
        $emailTemplateVariables = [];
        $emailTempVariables['cron'] = 'Price Check';

        $tableBodyPrices = '';
        $tableBodyCategory = '';
        $tableBodyImages = '';

        foreach ($data as $key => $values) {
            foreach ($values as $value => $stores) {
                if ($value == 'price') {
                    $tableBodyPrices .= '<tr>';
                    $tableBodyPrices .= '<td>' . $key . '</td>';
                    $tableBodyPrices .= '<td>Price less than 0</td>';
                    $tableBodyPrices .= '<td>' . implode(', ', array_keys($stores)) . '</td>';
                    $tableBodyPrices .= '</tr>';
                } elseif ($value == 'price_configurable') {
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>' . $key . '</td>';
                    $tableBodyCategory .= '<td>All children have zero price</td>';
                    $tableBodyCategory .= '<td>' . implode(', ', array_keys($stores)) . '</td>';
                    $tableBodyCategory .= '</tr>';
                } elseif ($value == 'category') {
                    $tableBodyCategory .= '<tr>';
                    $tableBodyCategory .= '<td>' . $key . '</td>';
                    $tableBodyCategory .= '<td>Without category</td>';
                    $tableBodyCategory .= '<td>' . implode(', ', array_keys($stores)) . '</td>';
                    $tableBodyCategory .= '</tr>';
                } elseif ($value == 'images') {
                    $tableBodyImages .= '<tr>';
                    $tableBodyImages .= '<td>' . $key . '</td>';
                    $tableBodyImages .= '<td>Image not found</td>';
                    $tableBodyImages .= '<td>' . implode(', ', array_keys($stores)) . '</td>';
                    $tableBodyImages .= '</tr>';
                }
            }
        }


        $emailTempVariables['tBodyPrices'] = $tableBodyPrices;
        $emailTempVariables['tBodyCategory'] = $tableBodyCategory;
        $emailTempVariables['tBodyImages'] = $tableBodyImages;

        /* call send mail method from helper or where you define it*/
        $this->sendCustomEmail(
            $emailTempVariables,
            $senderInfo,
            $receiversEmail,
            'filoblu_checkproducts/checkproducts_email/checkproducts_email_template_price'
        );
    }

    /**
     * @param $emailTemplateVariables
     * @param $senderInfo
     * @param $receiverInfo
     * @param $templateId
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\MailException
     */
    public function sendCustomEmail($emailTemplateVariables, $senderInfo, $receiverInfo, $templateId = null)
    {
        if (!$templateId) {
            $this->temp_id = $this->getTemplateId(self::XML_PATH_EMAIL_TEMPLATE_FIELD);
        } else {
            $this->temp_id = $this->getTemplateId($templateId);
        }
        $this->inlineTranslation->suspend();
        $this->generateTemplate($emailTemplateVariables, $senderInfo, $receiverInfo);
        $transport = $this->transportBuilder->getTransport();
        $transport->sendMessage();
        $this->inlineTranslation->resume();
    }

    /**
     * Return template id according to store
     *
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getTemplateId($xmlPath)
    {
        return $this->getConfigValue($xmlPath, $this->getStore()->getStoreId());
    }

    /**
     * Return store configuration value of your template field that which id you set for template
     *
     * @param string $path
     * @param int $storeId
     * @return mixed
     */
    protected function getConfigValue($path, $storeId)
    {
        return $this->scopeConfig->getValue(
            $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * [sendInvoicedOrderEmail description]
     * @param Mixed $emailTemplateVariables
     * @param Mixed $senderInfo
     * @param Mixed $receiverInfo
     * @return void
     */
    /* your send mail method*/

    /**
     * Return store
     *
     * @return \Magento\Store\Api\Data\StoreInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getStore()
    {
        return $this->storeManager->getStore();
    }

    /**
     * [generateTemplate description]  with template file and tempaltes variables values
     * @param Mixed $emailTemplateVariables
     * @param Mixed $senderInfo
     * @param Mixed $receiverInfo
     * @return \FiloBlu\Checkproducts\Helper\Email
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function generateTemplate($emailTemplateVariables, $senderInfo, $receiverInfo)
    {
        $template = $this->transportBuilder->setTemplateIdentifier($this->temp_id)
            ->setTemplateOptions(
                [
                    'area'  => Area::AREA_FRONTEND, /* here you can defile area and
                                                                                 store of template for which you prepare it */
                    'store' => $this->storeManager->getStore()->getId(),
                ]
            )
            ->setTemplateVars($emailTemplateVariables)
            ->setFrom($senderInfo)
            ->addTo($receiverInfo, '');

        return $this;
    }

}